﻿using DotNetCore.CAP;
using MapsterMapper;
using MediatR;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Push.Share.Dtos;
using Push.Share.Dtos.FWMessage;
using Push.Share.Enums;
using Push.YiBin.Dtos;
using System.Net;
using System.Xml;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Push.YiBin;

/// <summary>
/// 
/// </summary>
public class PushDomainService : IPushDomainService, IScopeDependency
{
    #region 注入

    private readonly IRepository<Message> _messageRepository;
    private readonly IMapper _mapper;
    private readonly IHttpClientFactory _httpClientFactory;
    private readonly IConfiguration _config;
    private readonly SmsAccountInfo? accountInfo = null;
    private readonly IMediator _mediator;
    private readonly ITypedCache<CacheWaitSendId> _cacheWaitSendId;
    private readonly ICapPublisher _capPublisher;
    private readonly ILogger<PushDomainService> _logger;
    private readonly IRepository<WaitMessage> _waitMessageRepository;
    private readonly FwClient _fwClient;

    /// <summary>
    /// 
    /// </summary>
    /// <param name="messageRepository"></param>
    /// <param name="httpClientFactory"></param>
    /// <param name="mapper"></param>
    /// <param name="config"></param>
    /// <param name="mediator"></param>
    /// <param name="cacheWaitSendId"></param>
    /// <param name="capPublisher"></param>
    public PushDomainService(IRepository<Message> messageRepository, IHttpClientFactory httpClientFactory
        , IMapper mapper, IConfiguration config, IMediator mediator,
        ITypedCache<CacheWaitSendId> cacheWaitSendId,
        ICapPublisher capPublisher,
        ILogger<PushDomainService> logger,
        IRepository<WaitMessage> waitMessageRepository,
       FwClient fwClient)
    {
        _messageRepository = messageRepository;
        _httpClientFactory = httpClientFactory;
        _mapper = mapper;
        _config = config;
        accountInfo = _config.GetSection("SmsAccountInfo").Get<SmsAccountInfo>();
        _mediator = mediator;
        _cacheWaitSendId = cacheWaitSendId;
        _capPublisher = capPublisher;
        _logger = logger;
        _waitMessageRepository = waitMessageRepository;
        _fwClient = fwClient;
    }

    #endregion

    #region 短信发送---新版
    /// <summary>
    /// 短信写入待发表
    /// </summary>
    /// <param name="dto"></param>
    /// <param name="cancellation"></param>
    /// <returns></returns>
    public async Task<Reponse> AddWaitMessage(WaitMessage dto, CancellationToken cancellation)
    {
        Reponse reponse = new Reponse() { Code = 1 };
        if (dto == null)
        {
            reponse.Message = "消息不能为空！";
            return reponse;
        }


        if (string.IsNullOrEmpty(dto.ClientId))
        {
            reponse.Message = "发送平台不能为空！";
            return reponse;
        }

        if (string.IsNullOrEmpty(dto.TelNumber))
        {
            reponse.Message = "短信接收号码不能为空！";
            return reponse;
        }

        if (string.IsNullOrEmpty(dto.Name))
        {
            reponse.Message = "接收姓名不能为空！";
            return reponse;
        }

        if (string.IsNullOrEmpty(dto.Content))
        {
            reponse.Message = "短信内容不能为空！";
            return reponse;
        }

        var id = await _waitMessageRepository.AddAsync(dto, cancellation);
        if (string.IsNullOrEmpty(id))
        {
            reponse.Message = "新增失败！";
            return reponse;
        }
        else
        {
            reponse.Code = 0;
            reponse.Message = "新增成功！";
            return reponse;

        }
    }

    /// <summary>
    /// 短信发送
    /// </summary>
    /// <param name="messageDto"></param>
    /// <returns></returns>
    public async Task<Message> PushNewAsync(Share.Dtos.MessageDto messageDto)
    {
        var message = _mapper.Map<Message>(messageDto);
        int WaitSendId = GenerateWaitSendId();
        message.WaitSendId = WaitSendId;

        _logger.LogInformation($"取到WaitSendId值：{WaitSendId}"+DateTime.Now);

        //调用短信发送业务
        var strResult = await SmsSend(messageDto, WaitSendId);

        //处理返回值
        if (true == strResult.Contains("ok,")) // ok,35797257
        {
            string[] strArray = strResult.Split(',');
            if (2 == strArray.Length)
            {
                // 短信服务中心返回 IFSFWID，用于短信发送状态接收时匹配
                message.SmsWaitSendingId = strArray[1];
                message.SendState = ESendState.Sending;
                message.Status = EPushStatus.Success;
            }
        }
        else
        {
            message.Reason = strResult;
            message.Status = EPushStatus.Failed;
        }
        return message;
    }
    #endregion

    #region 短信发送

    /// <summary>
    /// 短信发送
    /// </summary>
    /// <param name="messageDto"></param>
    /// <param name="cancellation"></param>
    /// <returns></returns>
    public async Task PushAsync(Share.Dtos.MessageDto messageDto, CancellationToken cancellation)
    {
        if (messageDto == null)
            throw UserFriendlyException.SameMessage("消息不能为空！");

        if (string.IsNullOrEmpty(messageDto.ClientId))
            throw UserFriendlyException.SameMessage("发送平台不能为空！");

        if (string.IsNullOrEmpty(messageDto.TelNumber))
            throw UserFriendlyException.SameMessage("短信接收号码不能为空！");

        if (string.IsNullOrEmpty(messageDto.Name))
            throw UserFriendlyException.SameMessage("接收姓名不能为空！");

        if (string.IsNullOrEmpty(messageDto.Content))
            throw UserFriendlyException.SameMessage("短信内容不能为空！");

        var message = _mapper.Map<Message>(messageDto);
        int WaitSendId = GenerateWaitSendId();
        message.WaitSendId = WaitSendId;

        _logger.LogInformation($"取到WaitSendId值：{WaitSendId}");

        //调用短信发送业务
        var strResult = await SmsSend(messageDto, WaitSendId);

        //处理返回值
        if (true == strResult.Contains("ok,")) // ok,35797257
        {
            string[] strArray = strResult.Split(',');
            if (2 == strArray.Length)
            {
                // 短信服务中心返回 IFSFWID，用于短信发送状态接收时匹配
                message.SmsWaitSendingId = strArray[1];
                message.SendState = ESendState.Sending;
                message.Status = EPushStatus.Success;
            }
        }
        else
        {
            message.Reason = strResult;
            message.Status = EPushStatus.Failed;
        }

        await _messageRepository.AddAsync(message, cancellation); //写入本地数据库

        //if (message.ClientId == "Hotline")
        //{
        //    var data = _mapper.Map<PushMessageDto>(message);
        //    data.Type = "0";
        //    await _capPublisher.PublishAsync(Push.Share.EventNames.UpdateSendSmsState, data, cancellationToken: cancellation);
        //}
    }

    #endregion

    #region 查询账号短信剩余量

    /// <summary>
    /// 查询账号短信剩余量
    /// </summary>
    /// <returns></returns>
    public async Task<string> GetAccountNum()
    {
        // 账户,密码
        string strAcceptContent = $"{accountInfo.AccountUser},{accountInfo.AccountPwd}";
        // 参数
        Dictionary<string, string> dicParam = new()
        {
            { "AcceptType", "fwAccountNum" },
            { "AcceptContent", strAcceptContent }
        };
        // 将参数转化为HttpContent
        HttpContent content = new FormUrlEncodedContent(dicParam);
        // ok,9
        // error,查询异常
        string strResult = await PostHelper(accountInfo.MessageServerUrl, content);
        var strArray = strResult.Split(",");
        if (2 == strArray.Length)
        {
            strResult = strArray[1];
        }
        else
        {
            strResult = "查询异常";
        }

        return strResult;
    }

    #endregion

    #region 短信发送状态回调接口

    /// <summary>
    /// 短信发送状态回调接口
    /// </summary>
    /// <param name="receiveMessageDto"></param>
    /// <returns></returns>
    public async Task<string> ReceiveObtain(ReceiveMessageDto receiveMessageDto)
    {
        string strResult = "error,缺少参数";
        if (receiveMessageDto.account != accountInfo.ReturnAccountUser || receiveMessageDto.pswd != accountInfo.ReturnAccountPwd)
        {
            strResult = "error,参数错误";
            return strResult;
        }

        if (true == string.IsNullOrEmpty(receiveMessageDto.account) || true == string.IsNullOrEmpty(receiveMessageDto.pswd)
            || true == string.IsNullOrEmpty(receiveMessageDto.msgid) || true == string.IsNullOrEmpty(receiveMessageDto.sendtime)
            || receiveMessageDto.msgcount <= 0 || receiveMessageDto.state < 0)
        {
            strResult = "error,参数错误";
            return strResult;
        }

        //修改数据
        var data = await _messageRepository.GetAsync(p => p.SmsWaitSendingId == receiveMessageDto.msgid);
        if (data != null)
        {
            data.SendTime = Convert.ToDateTime(receiveMessageDto.sendtime);
            data.MsgCount = receiveMessageDto.msgcount;
            data.SendState = (ESendState)receiveMessageDto.state;
            data.SmsSendingCompletedId = receiveMessageDto.sfid + "";
            data.Reason = receiveMessageDto.errormsg;
            await _messageRepository.UpdateAsync(data);

            try
            {
                if (data.ClientId == "Hotline")
                {
                    var dataPush = _mapper.Map<PushMessageDto>(data);
                    dataPush.Type = "1";
                    await _fwClient.RequestNoTokenAsync<Reponse>("api/v1/PushMessage/update-send-sms-state", "Post", System.Text.Json.JsonSerializer.Serialize(dataPush), cancellationToken: default);
                }
            }
            catch (Exception)
            {

            }
        }
        // 成功返回值必须是ok
        strResult = "ok";

        return strResult;
    }

    #endregion

    #region 短信接收

    /// <summary>
    /// 短信接收
    /// </summary>
    /// <param name="receiveMessageDto"></param>
    /// <returns></returns>
    public async Task<string> ReceiveSms(ReceiveMessageDto receiveMessageDto)
    {
        string strResult = "error,缺少参数";
        if (receiveMessageDto.account != accountInfo.ReturnAccountUser || receiveMessageDto.pswd != accountInfo.ReturnAccountPwd)
        {
            strResult = "error,参数错误";
            return strResult;
        }

        if (true == string.IsNullOrEmpty(receiveMessageDto.account) || true == string.IsNullOrEmpty(receiveMessageDto.pswd)
            || true == string.IsNullOrEmpty(receiveMessageDto.msg) || true == string.IsNullOrEmpty(receiveMessageDto.mobile)
            || true == string.IsNullOrEmpty(receiveMessageDto.motime))
        {
            strResult = "error,参数错误";
            return strResult;
        }

        var data = await _messageRepository.GetAsync(p => p.TelNumber == receiveMessageDto.mobile && p.IsSmsReply == false && p.SmsReplyTime == null);
        if (data != null)
        {
            data.IsSmsReply = true;
            data.SmsReplyTime = Convert.ToDateTime(receiveMessageDto.motime);
            data.SmsReplyContent = receiveMessageDto.msg;
            await _messageRepository.UpdateAsync(data);
            try
            {
                if (data.ClientId == "Hotline")
                {
                    var dataPush = _mapper.Map<PushMessageDto>(data);
                    dataPush.Type = "2";
                    await _fwClient.RequestNoTokenAsync<Reponse>("api/v1/PushMessage/update-send-sms-state", "Post", System.Text.Json.JsonSerializer.Serialize(dataPush), cancellationToken: default);
                }
            }
            catch (Exception)
            {

            }
        }

        // 成功返回值必须是ok
        strResult = "ok";

        return strResult;
    }

    #endregion

    #region 私有方法

    #region 短信发送

    /// <summary>
    /// 短信发送
    /// </summary>
    /// <returns></returns>
    private async Task<string> SmsSend(Share.Dtos.MessageDto messageDto, int WaitSendId)
    {
        _logger.LogInformation("准备短信推送---------------------"+DateTime.Now);
        string strResult;
        try
        {
            SendSmsModelDto tSms = new()
            {
                // 业务系统短信ID
                nWaitID = WaitSendId,
                // 短信类型1:默认2:营销
                strType = "1",
                // 业务系统工单ID
                nFlowID = 0,
                // 姓名
                strPhoneName = messageDto.Name,
                // 手机号码
                strPhoneNumber = messageDto.TelNumber,
                // 短信内容
                strPhoneConnent = messageDto.Content,
                // 添加时间
                dAddTime = DateTime.Now,
                strPhoneNumberAll = "",
                strRemark = "",
                // 业务系统添加人ID
                nUserID = 0,
                // 业务系统添加人姓名
                strUserName = "",
                // 业务系统添加部门ID
                nBMID = 0,
                // 业务系统添加部门名称
                strBMName = "",
                // 短信系统账号
                strAccountUser = accountInfo.AccountUser,
                // 短信系统密码
                strAccountPwd = accountInfo.AccountPwd
            };
            // 序列化
            string strSendJson = Newtonsoft.Json.JsonConvert.SerializeObject(tSms);

            // 参数加密
            string strKey = await FwEncrypt(strSendJson);
            // 短信发送参数
            Dictionary<string, string> dicParam = new Dictionary<string, string>
            {
                { "AcceptType", "fwSms" },
                { "AcceptContent", strKey }
            };
            // 将参数转化为HttpContent
            HttpContent content = new FormUrlEncodedContent(dicParam);
            strResult = await PostHelper(accountInfo.MessageServerUrl, content);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex.InnerException?.Message);
            strResult = ex.Message;
        }

        return strResult;
    }

    #endregion

    #region 参数加密

    /// <summary>
    /// 参数加密
    /// </summary>
    /// <param name="planText"></param>
    /// <returns></returns>
    private async Task<string> FwEncrypt(string planText)
    {
        // 参数
        Dictionary<string, string> dicParam = new Dictionary<string, string>();
        dicParam.Add("AcceptType", "fwEncrypt");
        dicParam.Add("AcceptContent", planText);
        // 将参数转化为HttpContent
        HttpContent content = new FormUrlEncodedContent(dicParam);
        string strResult = await PostHelper(accountInfo.MessageServerUrl, content);
        string strValue = "";
        // ok,XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
        if (true == strResult.Contains("ok,"))
        {
            string[] strArray = strResult.Split(',');
            if (2 == strArray.Length)
            {
                strValue = strArray[1];
            }
        }

        return strValue;
    }

    #endregion

    #region http请求

    /// <summary>
    /// 封装使用HttpClient调用WebService
    /// </summary>
    /// <param name="url">URL地址</param>
    /// <param name="content">参数</param>
    /// <returns></returns>
    private async Task<string> PostHelper(string url, HttpContent content)
    {
        _logger.LogInformation($"准备推送短信, {nameof(PostHelper)}"+DateTime.Now);
        var result = string.Empty;
        try
        {
            using var client = _httpClientFactory.CreateClient();
            using (var response = await client.PostAsync(url, content))
            {
                if (response.StatusCode == HttpStatusCode.OK)
                {
                    result = await response.Content.ReadAsStringAsync();
                    XmlDocument doc = new XmlDocument();
                    doc.LoadXml(result);
                    result = doc.InnerText;
                }
            }
        }
        catch (Exception ex)
        {
            _logger.LogError(ex.InnerException?.Message);
            result = ex.Message;
        }
        _logger.LogInformation($"推送响应：{result}");
        return result;
    }

    #endregion

    #region 生成待发短信ID缓存

    /// <summary>
    /// 生成待发短信ID缓存
    /// </summary>
    /// <returns></returns>
    private int GenerateWaitSendId()
    {
        var cacheKey = "PushMessageWaitSendId";
        var cacheWaitSend = _cacheWaitSendId.GetOrSet(cacheKey, f =>
        {
            var waitSendId = _messageRepository.Queryable()
                .MaxAsync(p => p.WaitSendId)
                .GetAwaiter()
                .GetResult();
            return new CacheWaitSendId { WaitSendId = waitSendId };
        }, TimeSpan.FromDays(1));
        cacheWaitSend!.WaitSendId += 1;

        _cacheWaitSendId.Set(cacheKey, cacheWaitSend);
        return cacheWaitSend.WaitSendId;
    }

    public class CacheWaitSendId
    {
        public int WaitSendId { get; set; }
    }

    #endregion

    #endregion
}