﻿using MapsterMapper;
using Microsoft.Extensions.Logging;
using Push.Share.Enums;
using Quartz;

namespace Push.YiBin
{
    public class TaskSendMessageJob : IJob, IDisposable
    {
        private readonly IMapper _mapper;
        private readonly ILogger<TaskSendMessageJob> _logger;
        private readonly IRepository<Message> _messageRepository;
        private readonly IRepository<WaitMessage> _waitMessageRepository;
        private readonly IPushDomainService _pushDomainService;
        private readonly IChannelConfigurationManager _channelConfigurationManager;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapper"></param>
        /// <param name="logger"></param>
        /// <param name="messageRepository"></param>
        /// <param name="waitMessageRepository"></param>
        /// <param name="pushDomainService"></param>
        /// <param name="channelConfigurationManager"></param>
        public TaskSendMessageJob(IMapper mapper,
            ILogger<TaskSendMessageJob> logger,
            IRepository<Message> messageRepository,
            IRepository<WaitMessage> waitMessageRepository,
            IPushDomainService pushDomainService,
           IChannelConfigurationManager channelConfigurationManager)
        {
            _mapper = mapper;
            _logger = logger;
            _messageRepository = messageRepository;
            _waitMessageRepository = waitMessageRepository;
            _pushDomainService = pushDomainService;
            _channelConfigurationManager = channelConfigurationManager;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        public async Task Execute(IJobExecutionContext context)
        {
           // _logger.LogWarning("进来了，时间是：" + DateTime.Now);
            var tasks = await _waitMessageRepository.Queryable()
                  .Where(d => d.Status == EPushStatus.Waiting && d.SendTimes <= 3)
                  .OrderBy(d => d.CreationTime)
                  .Take(50)
                  .ToListAsync(context.CancellationToken);
            if (tasks.Count != 0)
            {
                foreach (var sendTask in tasks)
                {
                    sendTask.Status = EPushStatus.Pushing;
                    if (await _waitMessageRepository.Updateable(sendTask).ExecuteCommandWithOptLockAsync() > 0)
                    {
                        _logger.LogWarning("执行前，时间是：" + DateTime.Now);
                        var result = await _pushDomainService.PushNewAsync(_mapper.Map<Share.Dtos.MessageDto>(sendTask));
                        _logger.LogWarning("执行后，时间是：" + DateTime.Now);
                        if (result.SendState == ESendState.Sending && result.Status == EPushStatus.Success)
                        {
                            //删除待推送表
                            await _waitMessageRepository.RemoveAsync(sendTask, cancellationToken: context.CancellationToken);

                            //修改添加推送数据
                            result.ProcessingServices = _channelConfigurationManager.GetConfigurationProcessingServices();
                            await _messageRepository.AddAsync(result, context.CancellationToken); //写入本地数据库
                        }
                        else
                        {
                            sendTask.SendTimes = sendTask.SendTimes + 1;
                            sendTask.Reason = result.Reason;
                            if (sendTask.SendTimes >= 4)
                                sendTask.Status = EPushStatus.Failed;
                            else
                                sendTask.Status = EPushStatus.Waiting;
                            await _waitMessageRepository.UpdateAsync(sendTask, context.CancellationToken);
                        }
                    }
                }
            }
        }


        public void Dispose()
        {

        }
    }
}
